import 'package:intl/intl.dart';

class EDateTime {
  static const int monday = 1;
  static const int tuesday = 2;
  static const int wednesday = 3;
  static const int thursday = 4;
  static const int friday = 5;
  static const int saturday = 6;
  static const int sunday = 7;
  static const int daysPerWeek = 7;

  static const int january = 1;
  static const int february = 2;
  static const int march = 3;
  static const int april = 4;
  static const int may = 5;
  static const int june = 6;
  static const int july = 7;
  static const int august = 8;
  static const int september = 9;
  static const int october = 10;
  static const int november = 11;
  static const int december = 12;
  static const int monthsPerYear = 12;

  late DateTime _date;

  EDateTime(int year,
      [int month = 1,
      int day = 1,
      int hour = 0,
      int minute = 0,
      int second = 0,
      int millisecond = 0,
      int microsecond = 0])
      : _date = DateTime(
            year, month, day, hour, minute, second, millisecond, microsecond);

  EDateTime.fromDateTime(DateTime date) : _date = date;

  EDateTime.utc(int year,
      [int month = 1,
      int day = 1,
      int hour = 0,
      int minute = 0,
      int second = 0,
      int millisecond = 0,
      int microsecond = 0])
      : _date = DateTime.utc(
            year, month, day, hour, minute, second, millisecond, microsecond);

  EDateTime.now() : _date = DateTime.now();

  EDateTime.fromMillisecondsSinceEpoch(int millisecondsSinceEpoch,
      {bool isUtc = false})
      : _date = DateTime.fromMillisecondsSinceEpoch(millisecondsSinceEpoch,
            isUtc: isUtc);

  EDateTime.fromMicrosecondsSinceEpoch(int microsecondsSinceEpoch,
      {bool isUtc = false})
      : _date = DateTime.fromMicrosecondsSinceEpoch(microsecondsSinceEpoch,
            isUtc: isUtc);

  int get day => _date.day;

  int get hour => _date.hour;

  bool get isUtc => _date.isUtc;

  int get microsecond => _date.microsecond;

  int get microsecondsSinceEpoch => _date.microsecondsSinceEpoch;

  int get millisecond => _date.millisecond;

  int get millisecondsSinceEpoch => _date.millisecondsSinceEpoch;

  int get minute => _date.minute;

  int get second => _date.second;

  int get weekday => _date.weekday;

  int get month => _date.month;

  String get timeZoneName => _date.timeZoneName;

  Duration get timeZoneOffset => _date.timeZoneOffset;

  int get year => _date.year;

static EDateTime parse(String date, [String? format]){
  return EDateTime.fromDateTime(DateFormat(format).parse(date));
}

  EDateTime add({
    int year = 0,
    int month = 0,
    int day = 0,
    int hour = 0,
    int minute = 0,
    int second = 0,
    int millisecond = 0,
    int microsecond = 0,
  }) {
    _date = DateTime(
        _date.year + year,
        _date.month + month,
        _date.day + day,
        _date.hour + hour,
        _date.minute + minute,
        _date.second + second,
        _date.millisecond + millisecond,
        _date.microsecond + microsecond);
    return EDateTime.fromDateTime(_date);
  }

  int compareTo(EDateTime other) {
    return _date.compareTo(other._date);
  }

  Duration difference(EDateTime other) {
    return _date.difference(other._date);
  }

  bool isAfter(EDateTime other) {
    return _date.isAfter(other._date);
  }

  bool isAtSameMomentAs(EDateTime other) {
    return _date.isAtSameMomentAs(other._date);
  }

  bool isBefore(EDateTime other) {
    return _date.isBefore(other._date);
  }

  EDateTime subtract({
    int year = 0,
    int month = 0,
    int day = 0,
    int hour = 0,
    int minute = 0,
    int second = 0,
    int millisecond = 0,
    int microsecond = 0,
  }) {
    _date = DateTime(
        _date.year - year,
        _date.month - month,
        _date.day - day,
        _date.hour - hour,
        _date.minute - minute,
        _date.second - second,
        _date.millisecond - millisecond,
        _date.microsecond - microsecond);
    return EDateTime.fromDateTime(_date);
  }

  // void subtract(Duration duration) {
  //   _date = _date.subtract(duration);
  // }

  String toIso8601String() {
    return _date.toIso8601String();
  }

  EDateTime toLocal() {
    return EDateTime.fromDateTime(_date.toLocal());
  }

  EDateTime toUtc() {
    return EDateTime.fromDateTime(_date.toUtc());
  }

  @override
  String toString() {
    return _date.toString();
  }

  String toFormatStr(String? format) {
    if (format == null) {
      return toString();
    } else {
      return DateFormat(format).format(_date);
    }
  }

  bool operator >(EDateTime other) {
    if (isAfter(other)) {
      return true;
    } else {
      return false;
    }
  }

  bool operator >=(EDateTime other) {
    if (isAfter(other) || _date == other._date) {
      return true;
    } else {
      return false;
    }
  }

  bool operator <=(EDateTime other) {
    if (isBefore(other) || _date == other._date) {
      return true;
    } else {
      return false;
    }
  }

  bool operator <(EDateTime other) {
    if (isBefore(other) || this == other) {
      return true;
    } else {
      return false;
    }
  }

  EDateTime operator -(EDateTime other) {
    return EDateTime.fromDateTime(_date.subtract(difference(other)));
  }

  @override
  bool operator ==(Object other) {
    return other is EDateTime && _date == other._date;
  }

  @override
  int get hashCode => _date.hashCode;
}
